using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;

namespace PCIe3521_NET
{
    /// <summary>
    /// Contains the call to the DLL functions
    /// </summary>
    public class DllImport
    {
        /// <summary>
        /// Definition of the function that must be given to use the interruption of the board
        /// </summary>
        /// <param name="handle"></param>
        /// <param name="fifoStatus"></param>
        /// <param name="source">Source of the interruption. 1: digital input, 2: digital output, 4: timer, 8: counter</param>
        /// <param name="mask"></param>
        /// <param name="size"></param>
        /// <param name="args"></param>
        [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
        public delegate void InterruptFunctionEventHandler(IntPtr handle,
                                                            byte overflow,
                                                            uint source,
                                                            uint mask,
                                                            IntPtr args,
                                                            uint argsSize
                                                            );

        [DllImport("PCIe3521.dll")]
        public static extern short i_PCIe3521_GetNumberOfBoards(ref byte boardNumber );

        [DllImport("PCIe3521.dll")]
        public static extern short i_PCIe3521_OpenBoardViaIndex(byte index, ref IntPtr deviceHandle );

        [DllImport("PCIe3521.dll")]
        public static extern short i_PCIe3521_CloseBoard(IntPtr deviceHandle);

        [DllImport("PCIe3521.dll")]
        public static extern short i_PCIe3521_SetBoardIntRoutine(IntPtr h_DeviceHandle, [MarshalAs(UnmanagedType.FunctionPtr)] InterruptFunctionEventHandler interruptFunction);

        [DllImport("PCIe3521.dll")]
        public static extern short i_PCIe3521_ResetBoardIntRoutine(IntPtr h_DeviceHandle);

        [DllImport("PCIe3521.dll")]
        public static extern short i_PCIe3521_SetDigitalOutputMemoryOn(IntPtr deviceHandle);

        [DllImport("PCIe3521.dll")]
        public static extern short i_PCIe3521_SetDigitalOutputMemoryOff(IntPtr deviceHandle);

        [DllImport("PCIe3521.dll")]
        public static extern short i_PCIe3521_Set1DigitalOutputOn(IntPtr deviceHandle, byte channel);

        [DllImport("PCIe3521.dll")]
        public static extern short i_PCIe3521_Set1DigitalOutputOff(IntPtr deviceHandle, byte channel);

        [DllImport("PCIe3521.dll")]
        public static extern short i_PCIe3521_Set4DigitalOutputsOn(IntPtr deviceHandle, byte value);

        [DllImport("PCIe3521.dll")]
        public static extern short i_PCIe3521_Set4DigitalOutputsOff(IntPtr deviceHandle, byte value);

        [DllImport("PCIe3521.dll")]
        public static extern short i_PCIe3521_Get1DigitalOutputStatus(IntPtr deviceHandle, byte channel, ref byte outputStatus);

        [DllImport("PCIe3521.dll")]
        public static extern short i_PCIe3521_Get4DigitalOutputsStatus(IntPtr deviceHandle, ref byte outputStatus);

        [DllImport("PCIe3521.dll")]
        public static extern short i_PCIe3521_Read1DigitalInput(IntPtr deviceHandle, byte channel, ref byte value);

        [DllImport("PCIe3521.dll")]
        public static extern short i_PCIe3521_Read4DigitalInputs(IntPtr deviceHandle, ref byte value);

        [DllImport("PCIe3521.dll")]
        public static extern short i_PCIe3521_InitAndStartTimer(IntPtr deviceHandle, byte timerNb, byte mode, byte timeBase, byte outputMode, byte interruptEnabled, ushort reloadValue, byte watchdogEnabled);

        [DllImport("PCIe3521.dll")]
        public static extern short i_PCIe3521_StopTimer(IntPtr deviceHandle, byte timerNb);

        [DllImport("PCIe3521.dll")]
        public static extern short i_PCIe3521_ReadTimerStatus(IntPtr deviceHandle, byte timerNb, ref byte status, ref byte softwareTriggerStatus, ref ushort value);

        [DllImport("PCIe3521.dll")]
        public static extern short i_PCIe3521_TriggerTimer(IntPtr deviceHandle, byte timerNb);

        [DllImport("PCIe3521.dll")]
        public static extern short i_PCIe3521_Write1AnalogOutput(IntPtr h_DeviceHandle, byte uc_Channel, byte uc_Polarity, ushort us_Value);

		[DllImport("PCIe3521.dll")]
        public static extern short i_PCIe3521_WriteMoreAnalogOutputs(IntPtr h_DeviceHandle, byte uc_ChannelNbr, byte[] puc_Channel, byte[] puc_Polarity, ushort[] pus_Value);
    }
}
